@@ -20,6 +20,7 @@ module Language.Preprocessor.Cpphs.Options
   ) where
 
 import Maybe
+import List (isPrefixOf)
 
 -- | Cpphs options structure.
 data CpphsOptions = CpphsOptions 
@@ -27,6 +28,7 @@ data CpphsOptions = CpphsOptions
     , outfiles	:: [FilePath]
     , defines	:: [(String,String)]
     , includes	:: [String]
+    , preInclude:: [FilePath]	-- ^ Files to #include before anything else
     , boolopts	:: BoolOptions
     }
 
@@ -34,16 +36,18 @@ data CpphsOptions = CpphsOptions
 defaultCpphsOptions :: CpphsOptions
 defaultCpphsOptions = CpphsOptions { infiles = [], outfiles = []
                                    , defines = [], includes = []
+                                   , preInclude = []
                                    , boolopts = defaultBoolOptions }
 
 -- | Options representable as Booleans.
 data BoolOptions = BoolOptions
     { macros	:: Bool  -- ^ Leave \#define and \#undef in output of ifdef?
-    , locations	:: Bool	 -- ^ Place \#line droppings in output?
-    , pragma	:: Bool  -- ^ Keep \#pragma in final output?
-    , strip	:: Bool  -- ^ Remove C comments everywhere?
+    , locations	:: Bool	 -- ^ Place #line droppings in output?
+    , pragma	:: Bool  -- ^ Keep #pragma in final output?
+    , stripEol	:: Bool  -- ^ Remove C eol (\/\/) comments everywhere?
+    , stripC89	:: Bool  -- ^ Remove C inline (\/**\/) comments everywhere?
     , lang	:: Bool  -- ^ Lex input as Haskell code?
-    , ansi	:: Bool  -- ^ Permit stringise # and catenate ## operators?
+    , ansi	:: Bool  -- ^ Permit stringise \# and catenate \#\# operators?
     , layout	:: Bool  -- ^ Retain newlines in macro expansions?
     , literate	:: Bool  -- ^ Remove literate markup?
     , warnings	:: Bool  -- ^ Issue warnings?
@@ -52,7 +56,8 @@ data BoolOptions = BoolOptions
 -- | Default settings of boolean options.
 defaultBoolOptions :: BoolOptions
 defaultBoolOptions = BoolOptions { macros   = True,   locations = True
-                                 , pragma   = False,  strip     = False
+                                 , pragma   = False
+                                 , stripEol = False,  stripC89  = False
                                  , lang     = True,   ansi      = False
                                  , layout   = False,  literate  = False
                                  , warnings = True }
@@ -65,12 +70,14 @@ data RawOption
     | Pragma
     | Text
     | Strip
+    | StripEol
     | Ansi
     | Layout
     | Unlit
     | SuppressWarnings
     | Macro (String,String)
     | Path String
+    | PreInclude FilePath
       deriving (Eq, Show)
 
 flags :: [(String, RawOption)]
@@ -79,6 +86,7 @@ flags = [ ("--nomacro", NoMacro)
         , ("--pragma",  Pragma)
         , ("--text",    Text)
         , ("--strip",   Strip)
+        , ("--strip-eol",  StripEol)
         , ("--hashes",  Ansi)
         , ("--layout",  Layout)
         , ("--unlit",   Unlit)
@@ -93,6 +101,8 @@ rawOption x | isJust a = a
 rawOption ('-':'D':xs) = Just $ Macro (s, if null d then "1" else tail d)
     where (s,d) = break (=='=') xs
 rawOption ('-':'I':xs) = Just $ Path $ trailing "/\\" xs
+rawOption xs | "--include="`isPrefixOf`xs
+            = Just $ PreInclude (drop 10 xs)
 rawOption _ = Nothing
 
 trailing :: (Eq a) => [a] -> [a] -> [a]
@@ -105,7 +115,8 @@ boolOpts opts =
     { macros	= not (NoMacro `elem` opts)
     , locations	= not (NoLine  `elem` opts)
     , pragma	=      Pragma  `elem` opts
-    , strip	=      Strip   `elem` opts
+    , stripEol	=      StripEol`elem` opts
+    , stripC89	=      StripEol`elem` opts || Strip `elem` opts
     , lang      = not (Text    `elem` opts)
     , ansi	=      Ansi    `elem` opts
     , layout	=      Layout  `elem` opts
@@ -127,6 +138,7 @@ parseOptions xs = f ([], [], []) xs
                            , outfiles = reverse outs
                            , defines  = [ x | Macro x <- reverse opts ]
                            , includes = [ x | Path x  <- reverse opts ]
+                           , preInclude=[ x | PreInclude x <- reverse opts ]
                            , boolopts = boolOpts opts
                            }
     normalise ('/':'/':filepath) = normalise ('/':filepath)
